@@ -79,26 +79,16 @@ class ISAMAppliance(IBMAppliance):
             json_data = json.loads(http_response.text)
             return_obj['data'] = json_data
         except ValueError:
-            return_obj['data'] = http_response.content.decode("utf-8")
-            return
-
-        self.logger.debug("Status Code: {0}".format(http_response.status_code))
-        if http_response.text != "":
-            self.logger.debug("Text: " + http_response.content.decode("utf-8"))
-
-        for key in http_response.headers:
-            if key == 'g-type':
-                if http_response.headers[key] == 'application/octet-stream; charset=UTF-8':
-                    json_data = {}
-                    return_obj.data = http_response.content.decode("utf-8")
-                    return
-
-        if http_response.text == "":
-            json_data = {}
-        else:
-            json_data = json.loads(http_response.text)
-
-        return_obj['data'] = json_data
+            try:
+                json_data = json.loads(http_response.content.decode("utf-8"))
+                return_obj['data'] = json_data
+            except UnicodeDecodeError:
+                return_obj['data'] = http_response.content
+            except ValueError:
+                if isinstance(http_response.content, bytes):
+                    return_obj['data'] = http_response.content.decode("utf-8")
+                else:
+                    return_obj['data'] = http_response.content
 
     def _process_connection_error(self, ignore_error, return_obj):
         if not ignore_error:
