@@ -26,14 +26,16 @@ def get(isamAppliance, check_mode=False, force=False):
 
 
 def set(isamAppliance, primary_master='127.0.0.1', secondary_master=None, master_ere=None, tertiary_master=None,
-        quaternary_master=None, dsc_external_clients=False, dsc_port=None, dsc_use_ssl=None, dsc_ssl_keyfile=None,
-        dsc_ssl_label=None, dsc_worker_threads=64, dsc_maximum_session_lifetime=3600, dsc_client_grace_period=600,
-        hvdb_embedded=True, hvdb_max_size=None, hvdb_db_type=None, hvdb_address=None, hvdb_port=None, hvdb_user=None,
-        hvdb_password=None, hvdb_db2_alt_address=None, hvdb_db2_alt_port=None, hvdb_db_name=None, hvdb_db_secure=None,
-        hvdb_driver_type=None, hvdb_solid_tc=None, cfgdb_embedded=True, cfgdb_db_type=None, cfgdb_address=None,
-        cfgdb_port=None, cfgdb_user=None, cfgdb_password=None, cfgdb_db2_alt_address=None, cfgdb_db2_alt_port=None,
-        cfgdb_db_name=None, cfgdb_db_secure=None, cfgdb_driver_type=None, cfgdb_solid_tc=None, first_port=2020,
-        cfgdb_fs=None, dsc_trace_level=None, ignore_password_for_idempotency=False, check_mode=False, force=False):
+       quaternary_master=None, dsc_external_clients=False, dsc_port=None, dsc_use_ssl=None, dsc_ssl_keyfile=None,
+       dsc_ssl_label=None, dsc_worker_threads=64, dsc_maximum_session_lifetime=3600, dsc_client_grace_period=600,
+       hvdb_embedded=True, hvdb_max_size=None, hvdb_db_type=None, hvdb_address=None, hvdb_port=None, hvdb_user=None,
+       hvdb_password=None, hvdb_db2_alt_address=None, hvdb_db2_alt_port=None, hvdb_db_name=None, hvdb_db_secure=None,
+       hvdb_driver_type=None, hvdb_solid_tc=None, cfgdb_embedded=True, cfgdb_db_type=None, cfgdb_address=None,
+       cfgdb_port=None, cfgdb_user=None, cfgdb_password=None, cfgdb_db2_alt_address=None, cfgdb_db2_alt_port=None,
+       cfgdb_db_name=None, cfgdb_db_secure=None, cfgdb_driver_type=None, cfgdb_solid_tc=None, first_port=2020,
+       cfgdb_fs=None, dsc_trace_level=None, cfgdb_db_truststore=None,
+       dsc_connection_idle_timeout=None, hvdb_failover_servers=None, cfgdb_failover_servers=None, hvdb_db_truststore=None,
+       ignore_password_for_idempotency=False, check_mode=False, force=False):
     """
     Set cluster configuration
     """
@@ -67,6 +69,8 @@ def set(isamAppliance, primary_master='127.0.0.1', secondary_master=None, master
         cluster_json["dsc_ssl_keyfile"] = dsc_ssl_keyfile
     if dsc_ssl_label is not None:
         cluster_json["dsc_ssl_label"] = dsc_ssl_label
+    if dsc_connection_idle_timeout is not None:
+        cluster_json["dsc_connection_idle_timeout"] = int(dsc_connection_idle_timeout)
     if hvdb_max_size is not None and hvdb_embedded:
         cluster_json["hvdb_max_size"] = int(hvdb_max_size)
     if hvdb_db_type is not None:
@@ -91,13 +95,24 @@ def set(isamAppliance, primary_master='127.0.0.1', secondary_master=None, master
         cluster_json["hvdb_db_name"] = hvdb_db_name
     if hvdb_db_secure is not None:
         cluster_json["hvdb_db_secure"] = bool(hvdb_db_secure)
-    if hvdb_driver_type is not None:
+    if hvdb_driver_type is not None and hvdb_db_type == "oracle":
+        #Oracle only
         cluster_json["hvdb_driver_type"] = hvdb_driver_type
     if hvdb_solid_tc is not None:
         if (isinstance(hvdb_solid_tc, basestring)):
             import ast
             hvdb_solid_tc = ast.literal_eval(hvdb_solid_tc)
         cluster_json["hvdb_solid_tc"] = hvdb_solid_tc
+    if hvdb_db_truststore is not None and hvdb_db_truststore != "":
+        cluster_json["hvdb_db_truststore"] = hvdb_db_truststore
+    if hvdb_failover_servers is not None:
+        #Only if the db_type is Postgresql.  This is an array [{"address": "postgresql2.domain.com", "port": 5432, "order": 0}]
+        if hvdb_db_type is not None and hvdb_db_type == "postgresql":
+            cluster_json["hvdb_failover_servers"] = hvdb_failover_servers
+    if cfgdb_failover_servers is not None:
+        #Only if the dbtype isPostgresql.  This is an array [{"address": "postgresql2.domain.com", "port": 5432, "order": 0}]
+        if cfgdb_db_type is not None and cfgdb_db_type == "postgresql":
+            cluster_json["cfgdb_failover_servers"] = cfgdb_failover_servers
     if cfgdb_db_type is not None:
         cluster_json["cfgdb_db_type"] = cfgdb_db_type
     if cfgdb_address is not None:
@@ -120,7 +135,7 @@ def set(isamAppliance, primary_master='127.0.0.1', secondary_master=None, master
         cluster_json["cfgdb_db_name"] = cfgdb_db_name
     if cfgdb_db_secure is not None:
         cluster_json["cfgdb_db_secure"] = bool(cfgdb_db_secure)
-    if cfgdb_driver_type is not None:
+    if cfgdb_driver_type is not None and cfgdb_db_type == "oracle":
         cluster_json["cfgdb_driver_type"] = cfgdb_driver_type
     if cfgdb_solid_tc is not None:
         if (isinstance(cfgdb_solid_tc, basestring)):
@@ -134,6 +149,8 @@ def set(isamAppliance, primary_master='127.0.0.1', secondary_master=None, master
                     isamAppliance.facts["version"], cfgdb_fs))
         else:
             cluster_json["cfgdb_fs"] = cfgdb_fs
+    if cfgdb_db_truststore is not None and cfgdb_db_truststore != "":
+        cluster_json["cfgdb_db_truststore"] = cfgdb_db_truststore
     if dsc_trace_level is not None:
         cluster_json["dsc_trace_level"] = dsc_trace_level
 
