@@ -3,7 +3,7 @@ import os.path
 import ibmsecurity.utilities.tools
 
 logger = logging.getLogger(__name__)
-requires_model="Appliance"
+requires_model = "Appliance"
 
 
 def get(isamAppliance, check_mode=False, force=False):
@@ -64,14 +64,15 @@ def _check_rollback(isamAppliance, fixpack):
 
     # Reverse sort the json by 'id'
     json_data_sorted = sorted(ret_obj['data'], key=lambda k: int(k['id']), reverse=True)
+
     # Eliminate all rollbacks before hitting the first non-rollback fixpack
     del_fixpack = ''  # Delete succeeding fixpack to a rollback, only last fixpack can be rolled back
-    for fixpack in json_data_sorted:
-        if fixpack['action'] == 'Uninstalled':
-            del_fixpack = fixpack['name']
-        elif del_fixpack == fixpack['name'] and fixpack['rollback'] == 'Yes':
+    for fp in json_data_sorted:
+        if fp['action'] == 'Uninstalled':
+            del_fixpack = fp['name']
+        elif del_fixpack == fp['name'] and fp['rollback'] == 'Yes':
             del_fixpack = ''
-        elif fixpack['name'].lower() == fixpack_name.lower():
+        elif fp['name'].lower() == fixpack_name.lower():
             return True
         # The first non-rollback fixpack needs to match the name otherwise skip rollback
         else:
@@ -90,17 +91,20 @@ def _check(isamAppliance, fixpack):
 
     # Reverse sort the json by 'id'
     json_data_sorted = sorted(ret_obj['data'], key=lambda k: int(k['id']), reverse=True)
-    # Eliminate all rollbacks
-    del_fixpack = ''  # Delete succeeding fixpack to a rollback, only last fixpack can be rolled back
-    for fixpack in json_data_sorted:
-        if fixpack['action'] == 'Uninstalled':
-            del_fixpack = fixpack['name']
-        elif del_fixpack == fixpack['name'] and fixpack['rollback'] == 'Yes':
-            del_fixpack = ''
-        elif fixpack['name'].lower() == fixpack_name.lower():
-            return True
 
-    return False
+    samename_fps = []
+    for fp in json_data_sorted:
+        if fp['name'] == fixpack_name:
+            samename_fps.append(fp)
+
+    if len(samename_fps) > 0:
+        fp_sorted = sorted(samename_fps, key=lambda k: int(k['id']), reverse=True)
+        if fp_sorted[0]['action'] == 'Installed':
+            return True
+        else:
+            return False
+    else:
+        return False
 
 
 def _extract_fixpack_name(fixpack):
@@ -120,9 +124,12 @@ def _extract_fixpack_name(fixpack):
     # Unable to extract fixpack name from binary
     # Return fixpack name derived from the filename
     file_name = os.path.basename(fixpack)
-    fixpack_name, ext_name = file_name.split('.')
-    logger.info("Fixpack name could not be extracted from binary, use name derived from the filename: {0}".format(fixpack_name))
-
+    if file_name.find(".fixpack"):
+        fixpack_name = file_name.partition(".fixpack")[0]
+    else:
+        fixpack_name, ext_name = file_name.split('.')
+    logger.info(
+        "Fixpack name could not be extracted from binary, use name derived from the filename: {0}".format(fixpack_name))
     return fixpack_name
 
 
