@@ -95,7 +95,8 @@ def set_file(isamAppliance, name, protocol, filename, mapping_id, role=None, tem
     # update mapping_id
     file_lines['identityMapping']['properties']['identityMappingRuleReference'] = str(mapping_id)
 
-    return set(isamAppliance, name, protocol, role, file_lines, templateName, new_name, check_mode, force)
+    return set(isamAppliance=isamAppliance, name=name, protocol=protocol, configuration=file_lines, role=role,
+               templateName=templateName, new_name=new_name, check_mode=check_mode, force=force)
 
 
 def add(isamAppliance, name, protocol, configuration, role=None, templateName=None, check_mode=False,
